Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports System.Diagnostics
Imports System.IO
Imports SyncCover.Properties

Public Class [Assembly]

	' public static Dictionary<string, string> env = new Dictionary<string, string>();

	Private ReadOnly ildasmPath As String
	Private ReadOnly ilasmPath As String

	Private ReadOnly assemblyPath As String
	Private ReadOnly assemblyName As String

	Private ReadOnly workingFolder As String

	Private ReadOnly ilFile As String
	Private ReadOnly ccIlFile As String
	Private ReadOnly assemblyLibPath As String

	Private lines_Renamed As List(Of String)

	Public Sub New(ByVal ildasmPath As String, ByVal ilasmPath As String, ByVal libPath As String, ByVal assemblyPath As String)
		Me.assemblyPath = assemblyPath
		Me.assemblyName = Path.GetFileName(assemblyPath)

		Me.ilasmPath = ilasmPath
		Me.ildasmPath = ildasmPath

		Me.workingFolder = Path.Combine(libPath, assemblyName)
		Me.assemblyLibPath = Path.Combine(workingFolder, assemblyName)

		Me.ilFile = Path.ChangeExtension(assemblyLibPath, "orig.il")
		Me.ccIlFile = Path.ChangeExtension(assemblyLibPath, ".il")

		CopyAssemblyAndPdb(assemblyPath, workingFolder)
	End Sub

	Public ReadOnly Property Name() As String
		Get
			Return assemblyName
		End Get
	End Property

	Public ReadOnly Property VersionInfo() As FileVersionInfo
		Get
			Return FileVersionInfo.GetVersionInfo(assemblyLibPath)
		End Get
	End Property

	Private Shared Sub CopyAssemblyAndPdb(ByVal srcAssembly As String, ByVal [to] As String)
		Dim fileName As String = Path.GetFileName(srcAssembly)

		Directory.CreateDirectory([to])
		File.Copy(srcAssembly, Path.Combine([to], fileName), True)
		File.Copy(Path.ChangeExtension(srcAssembly, ".pdb"), Path.Combine([to], Path.ChangeExtension(fileName, ".pdb")), True)
	End Sub

	Public ReadOnly Property Lines() As List(Of String)
		Get
			If lines_Renamed Is Nothing Then
				ReadLines()
			End If
			Return lines_Renamed
		End Get
	End Property

	Private Sub ReadLines()
		Decompile()
		If (Not File.Exists(ilFile)) Then
			Throw New FileNotFoundException("ilFile", ilFile)
		End If

		Using sr As New StreamReader(ilFile, Encoding.UTF8)
			lines_Renamed = New List(Of String)(ReadAllLines(sr))
		End Using
	End Sub

	Public Sub Decompile()
		If (Not File.Exists(assemblyLibPath)) Then
			Throw New FileNotFoundException("assemblyLibPath", assemblyLibPath)
		End If

		Dim ildasmArg As String = String.Format("{0} /out={1} /typelist /linenum /UTF8", assemblyLibPath, ilFile)

		Console.WriteLine("Decompiling: {0}", assemblyName)

		Dim stdOut, stdErr As String
		Dim exitCode As Integer = ProcessUtil.StartAndWaitForResult(ildasmPath, ildasmArg, stdOut, stdErr)
		If exitCode <> 0 Then
			Throw New Exception(stdErr)
		End If
	End Sub

	Friend Sub Compile(ByVal peverify As PeVerify, ByVal outPath As String)
		Dim outAssembly As String = Path.Combine(outPath, assemblyName)

		Dim isDll As Boolean = Path.GetExtension(assemblyName) = ".dll"
		Dim ilasmArgs As String = ccIlFile & (If(isDll, " /dll ", " ")) & "/OUTPUT=" & outAssembly & " /pdb"

		Dim stdOut, stdErr As String

		Console.WriteLine("Compiling: {0}", assemblyName)
		Dim exitCode As Integer = ProcessUtil.StartAndWaitForResult(ilasmPath, ilasmArgs, stdOut, stdErr)
		If exitCode <> 0 Then
			Throw New Exception(stdErr)
		End If

		Dim expectedResult = peverify.VerifyAssembly(assemblyPath)
		Dim result = peverify.VerifyAssembly(outAssembly)

		expectedResult.Errors.ForEach(Function(e) result.Errors.Remove(e))

		Console.WriteLine()
        result.Errors.ForEach(Function(a) WriteLine(a))
		Console.WriteLine()

		File.WriteAllLines("result.txt", result.Errors.ToArray())
		File.WriteAllLines("expectedResult.txt", expectedResult.Errors.ToArray())
    End Sub

    Private Function WriteLine(ByVal message As String) As Object
        Console.WriteLine(message)
        Return Nothing
    End Function

    Private Shared Function ReadAllLines(ByVal sr As StreamReader) As IEnumerable(Of String)
        Dim lines As New List(Of String)
        Do While Not sr.EndOfStream
            lines.Add(sr.ReadLine())
        Loop
        Return lines
    End Function

	Friend Sub WriteIL()
		Using sw As New StreamWriter(ccIlFile)
            Lines.ForEach(Function(l) WriteLine(sw, l))
		End Using
    End Sub

    Private Function WriteLine(ByVal writer As StreamWriter, ByVal message As String) As Object
        Return Nothing
    End Function

End Class ' class
