Imports Microsoft.VisualBasic
Imports System
Imports System.IO
Imports System.Diagnostics
Imports System.Collections.Generic
Imports System.Threading
Imports System.Text.RegularExpressions
Imports System.Collections.ObjectModel

Public Class Cover

	Private coverPointId As Integer = 0
	Private lines As List(Of String)
	Private ReadOnly data_Renamed As List(Of CoverData)

	Private currentIndex As Integer
	Private lastLine As Integer = 0
	Private inMethod As Boolean = False
	Private lastFile As String = String.Empty
	Private methodName As String = String.Empty
	Private inClass As String = String.Empty

	Private didInject As Boolean = False

    Private ReadOnly magicMap As Dictionary(Of String, String)

	Public Sub New()
        Me.data_Renamed = New List(Of CoverData)()
        magicMap = New Dictionary(Of String, String)
        With magicMap
            .Add("void [mscorlib]System.Threading.Monitor::Enter(object)", "void System.Threading.MyMonitor::Enter(object, int32)")
            .Add("void [mscorlib]System.Threading.Monitor::Enter(object, bool&)", "void System.Threading.MyMonitor::Enter(object, bool&, int32)")
        End With
	End Sub

	Public ReadOnly Property Data() As ReadOnlyCollection(Of CoverData)
		Get
			Return data_Renamed.AsReadOnly()
		End Get
	End Property

	Protected Sub OnCall(ByVal targetMethod As String)
		Dim replaceString As String

		If lastFile.Contains("MyMonitor.cs") Then
			Return
		End If

		If magicMap.TryGetValue(targetMethod, replaceString) Then
			didInject = True

			lines(currentIndex) = lines(currentIndex).Replace(targetMethod, replaceString)

			lines.Insert(currentIndex, String.Format("ldc.i4 {0}", coverPointId))
			currentIndex += 1

			data_Renamed.Add(New CoverData(coverPointId, lastLine, lastFile, inClass, methodName))
			coverPointId += 1

		ElseIf lines(currentIndex).Contains("System.Threading.Monitor::Enter") Then
			' this is the list of methods variants we didn't cover.
			File.AppendAllText("oops.txt", String.Format("{0}|{2}|{1}", currentIndex, targetMethod, lastFile) & Environment.NewLine)
		End If
	End Sub

	Private Function IsInClass(ByVal classFullName As String) As Boolean
		Return String.Equals(inClass, classFullName, StringComparison.CurrentCulture)
	End Function

	Public Sub InstrumentAssembly(ByVal lines As List(Of String))
		Me.lines = lines
		currentIndex = 0
		lastLine = 0
		inMethod = False
		lastFile = String.Empty
		methodName = String.Empty
		inClass = String.Empty

		Dim stackLine As Integer = -1

		For i As Integer = 0 To lines.Count - 1
			If inMethod Then
				If lines(i).Contains("// end of method") Then
					If didInject Then
						Dim x As String = lines(stackLine)
						Dim parts() As String = x.Split(New Char() { " "c }, StringSplitOptions.RemoveEmptyEntries)
						Dim y As Integer = Integer.Parse(parts(1))
						lines(stackLine) = String.Format(".maxstack {0}", y + 1)
					End If

					didInject = False
					inMethod = False
					stackLine = -1
				ElseIf lines(i).Contains(".maxstack ") Then
					stackLine = i

				ElseIf lines(i).Contains("call ") Then
					currentIndex = i

					Dim j As Integer = 1
					Do While lines(i).EndsWith(",")
						lines(i) &= " " & lines(i + j).TrimStart()
						lines(i + j) = "/* MOVED UP!!! */"
						j += 1
					Loop

					Dim startIndex As Integer = lines(currentIndex).IndexOf("call ")
					Dim targetMethod As String = lines(currentIndex).Substring(startIndex + 5).Trim()

					OnCall(targetMethod)

					i = currentIndex

				ElseIf lines(i).Contains("callvirt ") Then
					currentIndex = i

					Dim j As Integer = 1
					Do While lines(i).EndsWith(",")
						lines(i) &= " " & lines(i + j).TrimStart()
						lines(i + j) = "/* MOVED UP!!! */"
						j += 1
					Loop

					Dim startIndex As Integer = lines(currentIndex).IndexOf("callvirt ")
					Dim targetMethod As String = lines(currentIndex).Substring(startIndex + 9).Trim()

					OnCall(targetMethod)

					i = currentIndex

				ElseIf lines(i).Contains(".s ") Then
					lines(i) = lines(i).Replace(".s ", " ")

#If true Then

				ElseIf lines(i).Contains(" ble.s ") Then
					lines(i) = lines(i).Replace("ble.s ", "ble ")

				ElseIf lines(i).Contains("leave.s ") Then
					lines(i) = lines(i).Replace("leave.s ", "leave ")

				ElseIf lines(i).Contains("bne.un.s ") Then
					lines(i) = lines(i).Replace("bne.un.s ", "bne.un ")

				ElseIf lines(i).Contains("bge.s ") Then
					lines(i) = lines(i).Replace("bge.s ", "bge ")

				ElseIf lines(i).Contains("beq.s ") Then
					lines(i) = lines(i).Replace("beq.s ", "beq ")

				ElseIf lines(i).Contains("bgt.s") Then
					lines(i) = lines(i).Replace("bgt.s", "bgt")

				ElseIf lines(i).Contains("br.s") Then
					lines(i) = lines(i).Replace("br.s", "br")

				ElseIf lines(i).Contains("blt.un.s") Then
					lines(i) = lines(i).Replace("blt.un.s", "blt.un")

				ElseIf lines(i).Contains("brfalse.s") Then
					lines(i) = lines(i).Replace("brfalse.s", "brfalse")

				ElseIf lines(i).Contains("blt.s") Then
					lines(i) = lines(i).Replace("blt.s", "blt")

				ElseIf lines(i).Contains("brtrue.s") Then
					lines(i) = lines(i).Replace("brtrue.s", "brtrue")
#End If
				ElseIf lines(i).Contains(".line") Then
					Dim l As String = lines(i)

					Dim startIndex As Integer = l.IndexOf(","c) + 1
					Dim endIndex As Integer = l.IndexOf(":"c, startIndex)

					lastLine = Integer.Parse(l.Substring(startIndex, endIndex - startIndex))

					startIndex = l.IndexOf("'"c, endIndex) + 1
					endIndex = l.IndexOf("'"c, startIndex)

					If endIndex > startIndex Then
						lastFile = l.Substring(startIndex, endIndex - startIndex).Replace("\\", "\")
					End If
				End If

			ElseIf lines(i).Contains(".class") Then
				Dim lastSpace As Integer = lines(i).LastIndexOf(" "c)
				inClass = lines(i).Substring(lastSpace + 1)

			ElseIf lines(i).Contains(".method") Then
				Dim decl As String = String.Empty

				Do While lines(i).Trim() <> "{"
					decl &= lines(i)
					i += 1
				Loop

				Dim endIndex As Integer = decl.IndexOf("("c)
				Dim startIndex As Integer = decl.LastIndexOf(" "c, endIndex) + 1 ' skip the space.

				inMethod = True
				methodName = decl.Substring(startIndex, endIndex - startIndex)
			End If
		Next i
	End Sub

	Friend Sub InjectMyMonitor(ByVal coverTag As String, ByVal lines As List(Of String))
		Dim idx As Integer = lines.FindIndex(Function(a) a.Contains("// =============== CLASS MEMBERS DECLARATION ==================="))
		If idx = -1 Then
			Throw New Exception("Injection Point Changed!")
		End If

		Inject(coverTag, lines, idx, "SyncCover.il.myMonitor.txt")
	End Sub

	Friend Sub Inject(ByVal coverTag As String, ByVal lines As List(Of String), ByVal idx As Integer, ParamArray ByVal resourceStreamNames() As String)
		Dim resourceAssembly As System.Reflection.Assembly = System.Reflection.Assembly.GetExecutingAssembly()

		Dim newCode As List(Of String) = New List(Of String)()

		For Each resourceStreamName As String In resourceStreamNames
			Using srcStream As Stream = resourceAssembly.GetManifestResourceStream(resourceStreamName)
				newCode.InsertRange(newCode.Count, ReplaceString(coverTag, ReadAllLines(srcStream)))
			End Using
		Next resourceStreamName

		lines.InsertRange(idx + 1, newCode)
	End Sub

    Private Shared Function ReplaceString(ByVal coverTag As String, ByVal iEnumerable As IEnumerable(Of String)) As IEnumerable(Of String)
        Dim lines As New List(Of String)
        For Each s As String In iEnumerable
            lines.Add(s.Replace("ldstr      ""CC_COVER_TAG_CC""", "ldstr      """ & coverTag & """"))
        Next s
        Return lines
    End Function

    Private Shared Function ReadAllLines(ByVal srcStream As Stream) As IEnumerable(Of String)
        Dim lines As New List(Of String)
        Using r As New StreamReader(srcStream)
            Do While Not r.EndOfStream
                lines.Add(r.ReadLine())
            Loop
        End Using
        Return lines
    End Function
End Class ' class
