Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.IO
Imports System.Text
Imports System.Text.RegularExpressions

Public Class PeVerifyResult

	Public ExitCode As Integer
	Public AssemblyName As String
	Public Errors As List(Of String)

	Private ReadOnly Shared offsetEx As New Regex("\[offset 0x.*\]")

	Public Function NormalizeErrorString(ByVal [error] As String) As String
		' Lets remove any path information.
        Dim p As String = Path.GetDirectoryName(AssemblyName)

		Dim b As New StringBuilder([error])
        If p.Length > 0 Then
            b.Replace(p, "<path>")
        End If
		b.Replace(Constants.vbCrLf, " ")

		Return offsetEx.Replace(b.ToString(), "[<offset/>]")
	End Function

End Class ' class

Public Class PeVerify

	Private Const PeVerifyExpectedExitCode As Integer = 0

	Private ReadOnly peVerifyPathv3 As String

	Public Sub New(ByVal peVerifyPathv3 As String)
		Me.peVerifyPathv3 = peVerifyPathv3
	End Sub

	Public Function VerifyAssembly(ByVal assemblyName As String) As PeVerifyResult

		Dim result As New PeVerifyResult()
		result.AssemblyName = assemblyName

		Dim stdOut, stdErr As String
		result.ExitCode = ProcessUtil.StartAndWaitForResult(peVerifyPathv3, assemblyName & " /IL /NOLOGO", stdOut, stdErr)

		ParseErrors(result, stdOut)

		Return result
	End Function

	Private Sub ParseErrors(ByVal result As PeVerifyResult, ByVal stdOut As String)

		result.Errors = New List(Of String)()

		Dim startIndex As Integer = 0
		Do While startIndex < stdOut.Length
			startIndex = stdOut.IndexOf("[IL]:", startIndex)
			If startIndex = -1 Then
				Exit Do
			End If

			Dim endIndex As Integer = stdOut.IndexOf("[IL]:", startIndex + 1)
			If endIndex = -1 Then
				' Look for the last line...
				endIndex = stdOut.IndexOf(Constants.vbCrLf, startIndex + 1)
			End If

			result.Errors.Add(result.NormalizeErrorString(stdOut.Substring(startIndex, endIndex - startIndex)))
			startIndex = endIndex
		Loop
	End Sub

	Public Shared Sub Assert(ByVal expectedResult As PeVerifyResult, ByVal actualResult As PeVerifyResult)
		For Each e In expectedResult.Errors
			actualResult.Errors.Remove(e)
		Next e

		' TODO: Ideally this will be with peverify v4.0, and this won't be needed.
		If actualResult.Errors.Count > expectedResult.Errors.Count Then
			Console.WriteLine("Expected:")
            expectedResult.Errors.ForEach(Function(msg) WriteLine(msg))
			Console.WriteLine()

			Console.WriteLine("Actual:")
            actualResult.Errors.ForEach(Function(msg) WriteLine(msg))
			Console.WriteLine("PeVerify: {0}", expectedResult.AssemblyName)

			Throw New Exception("PeVerify Failed with " & actualResult.Errors.Count & " different errors.")
        End If
    End Sub

    Private Shared Function WriteLine(ByVal message As String) As Object
        Console.WriteLine(message)
        Return Nothing
    End Function
End Class ' class
