Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.IO
Imports System.Linq
Imports SyncCover.Properties

Friend Class Program

	Shared Function Main(ByVal args() As String) As Integer

        Dim settings = My.Settings.Default

		If (Not File.Exists(settings.IlasmPath)) Then
			Throw New FileNotFoundException("ilasm", settings.IlasmPath)
		ElseIf (Not File.Exists(settings.IldasmPath)) Then
			Throw New FileNotFoundException("ildasm", settings.IldasmPath)
		ElseIf (Not File.Exists(settings.PeVerifyPath)) Then
			Throw New FileNotFoundException("peverify", settings.PeVerifyPath)
		ElseIf args.Length <> 2 Then
			Console.WriteLine("Usage: <assemblyName> <versionTag>")
			Return 1
		End If

		Dim assemblyPath As String = args(0)

		If (Not Path.IsPathRooted(assemblyPath)) Then
			assemblyPath = Path.Combine(Environment.CurrentDirectory, assemblyPath)
		End If

		If (Not File.Exists(assemblyPath)) Then
			Throw New Exception(String.Format("Can't find assembly to cover. [{0}]", assemblyPath))
		End If

		Dim versionTag As String = args(1)
		Console.WriteLine("assemblyPath:{0} versionTag:{1}", assemblyPath, versionTag)

		Dim libRoot As String = Path.GetDirectoryName(assemblyPath)
		Dim libPath As String = Path.Combine(libRoot, "lib\")
		Dim outPath As String = Path.Combine(libRoot, "out\")
		Dim mapFile As String = Path.Combine(outPath, String.Format("cc.map.{0}.txt", versionTag))

		If (Not Directory.Exists(libPath)) Then
			Directory.CreateDirectory(libPath)
		End If

		If (Not Directory.Exists(outPath)) Then
			Directory.CreateDirectory(outPath)
		End If

		Dim c As New Cover()
		Dim [assembly] = New [Assembly](settings.IldasmPath, settings.IlasmPath, libPath, assemblyPath)

		Console.WriteLine("Instrumenting: " & [assembly].VersionInfo.FileName)
		c.InstrumentAssembly([assembly].Lines)
		c.InjectMyMonitor(versionTag, [assembly].Lines)

		[assembly].WriteIL()
		[assembly].Compile(New PeVerify(settings.PeVerifyPath), outPath)

		CoverMap.ToFile(mapFile, versionTag, c.Data)

		Console.WriteLine("mapFile: " & mapFile)
		Console.WriteLine("data.Count: " & c.Data.Count)

		Return 0
	End Function

End Class ' class
