Imports Microsoft.VisualBasic
Imports System.Collections.Generic
Imports System.IO
Imports System
Imports System.Diagnostics
Imports System.Text
Imports System.Globalization

Public Class SyncPoint

	Private privateSyncId As Integer
	Public Property SyncId() As Integer
		Get
			Return privateSyncId
		End Get
		Set(ByVal value As Integer)
			privateSyncId = value
		End Set
	End Property
	Private privateLineNumber As Integer
	Public Property LineNumber() As Integer
		Get
			Return privateLineNumber
		End Get
		Set(ByVal value As Integer)
			privateLineNumber = value
		End Set
	End Property

End Class ' class

Public Class CoverMapRow

	Private privateFileName As String
	Public Property FileName() As String
		Get
			Return privateFileName
		End Get
		Set(ByVal value As String)
			privateFileName = value
		End Set
	End Property
	Private privateSyncId As Integer
	Public Property SyncId() As Integer
		Get
			Return privateSyncId
		End Get
		Set(ByVal value As Integer)
			privateSyncId = value
		End Set
	End Property
	Private privateLineNumber As Integer
	Public Property LineNumber() As Integer
		Get
			Return privateLineNumber
		End Get
		Set(ByVal value As Integer)
			privateLineNumber = value
		End Set
	End Property

	Private privateClassName As String
	Public Property ClassName() As String
		Get
			Return privateClassName
		End Get
		Set(ByVal value As String)
			privateClassName = value
		End Set
	End Property
	Private privateMethodName As String
	Public Property MethodName() As String
		Get
			Return privateMethodName
		End Get
		Set(ByVal value As String)
			privateMethodName = value
		End Set
	End Property

	Private privateTag As String
	Public Property Tag() As String
		Get
			Return privateTag
		End Get
		Set(ByVal value As String)
			privateTag = value
		End Set
	End Property

End Class ' class

Public Class CoverData

	Public Id As Integer
	Public LineNumber As Integer
	Public FileName As String
	Public ClassName As String
	Public MethodName As String

	Public Sub New(ByVal id As Integer, ByVal lineNumber As Integer, ByVal fileName As String, ByVal className As String, ByVal methodName As String)
		Me.Id = id
		Me.LineNumber = lineNumber
		Me.FileName = fileName
		Me.ClassName = className
		Me.MethodName = methodName
	End Sub

	Public Overrides Function ToString() As String
		Return String.Format("{0}|{1}|{2}|{3}|{4}", Id, FileName, LineNumber, ClassName, MethodName)
	End Function

End Class ' class

Public Class CoverMap

	Public Shared Sub ToFile(ByVal toFile As String, ByVal coverTag As String, ByVal data As IEnumerable(Of CoverData))
		Using w As New StreamWriter(toFile)
			w.WriteLine("T|" & coverTag)
			w.WriteLine("SP|")
			For Each d In data
				w.WriteLine(d)
			Next d
		End Using
	End Sub

	Public Shared Function FromFile(ByVal fileName As String) As IEnumerable(Of CoverMapRow)
		Using sr As New StreamReader(fileName)

			Dim versionTag As String = CoverRun.ExtractField(sr.ReadLine(), "T")
			CoverRun.EatUntil(sr, "SP")
            Dim mapRows As New List(Of CoverMapRow)
			Do While Not sr.EndOfStream
				Dim parts() As String = sr.ReadLine().Split("|"c)
				Debug.Assert(parts.Length = 5)

				Dim theTag As String = CreateTagFromFileName(parts(1))

                mapRows.Add(New CoverMapRow() With { _
                                .SyncId = Integer.Parse(parts(0)), _
                                .FileName = parts(1), _
                                .LineNumber = Integer.Parse(parts(2)), _
                                .ClassName = parts(3), _
                                .MethodName = parts(4), _
                                .Tag = theTag})
            Loop
            Return mapRows
		End Using
	End Function

	Private Shared Function CreateTagFromFileName(ByVal p As String) As String
		Return p
	End Function

	Public Shared Function GetFileNameFromTag(ByVal versionTag As String) As String
		Return Path.Combine(Environment.CurrentDirectory, String.Format("cc.map.{0}.txt", versionTag))
	End Function

End Class ' class

Public Enum Coverage
	None = 0
	Statement = 1
	Sync = 2
End Enum

Public Class RunData
	Private privateSyncId As Integer
	Public Property SyncId() As Integer
		Get
			Return privateSyncId
		End Get
		Set(ByVal value As Integer)
			privateSyncId = value
		End Set
	End Property
	Private privateHit As Long
	Public Property Hit() As Long
		Get
			Return privateHit
		End Get
		Set(ByVal value As Long)
			privateHit = value
		End Set
	End Property
	Private privateContend As Long
	Public Property Contend() As Long
		Get
			Return privateContend
		End Get
		Set(ByVal value As Long)
			privateContend = value
		End Set
	End Property

	Public ReadOnly Property DidWait() As Boolean
		Get
			Return Contend <> 0
		End Get
	End Property

	Public ReadOnly Property WasEntered() As Boolean
		Get
			Return Hit <> 0 OrElse Contend <> 0
		End Get
	End Property

	Public ReadOnly Property Coverage() As Coverage
		Get
			If DidWait Then
				Return Coverage.Sync
			End If
			If WasEntered Then
				Return Coverage.Statement
			End If
			Return Coverage.None
		End Get
	End Property
End Class

Public Class CoverRun

	Private privateProgram As String
	Public Property Program() As String
		Get
			Return privateProgram
		End Get
		Set(ByVal value As String)
			privateProgram = value
		End Set
	End Property
	Private privateCommandLine As String
	Public Property CommandLine() As String
		Get
			Return privateCommandLine
		End Get
		Set(ByVal value As String)
			privateCommandLine = value
		End Set
	End Property

	Private privateLogTime As DateTime
	Public Property LogTime() As DateTime
		Get
			Return privateLogTime
		End Get
		Set(ByVal value As DateTime)
			privateLogTime = value
		End Set
	End Property
	Private privateVersionTag As String
	Public Property VersionTag() As String
		Get
			Return privateVersionTag
		End Get
		Set(ByVal value As String)
			privateVersionTag = value
		End Set
	End Property
	Private privateData As IList(Of RunData)
	Public Property Data() As IList(Of RunData)
		Get
			Return privateData
		End Get
		Set(ByVal value As IList(Of RunData))
			privateData = value
		End Set
	End Property

	Private privateMachineName As String
	Public Property MachineName() As String
		Get
			Return privateMachineName
		End Get
		Set(ByVal value As String)
			privateMachineName = value
		End Set
	End Property
	Private privateOS As String
	Public Property OS() As String
		Get
			Return privateOS
		End Get
		Set(ByVal value As String)
			privateOS = value
		End Set
	End Property
	Private privateProcCount As Integer
	Public Property ProcCount() As Integer
		Get
			Return privateProcCount
		End Get
		Set(ByVal value As Integer)
			privateProcCount = value
		End Set
	End Property

	Public Shared Function FromFile(ByVal fileName As String) As IEnumerable(Of CoverRun)
        Dim runs As New List(Of CoverRun)
        Using sr As New StreamReader(fileName)

            Do While Not sr.EndOfStream
                Dim line As String = sr.ReadLine()
                If line = "~|B|~" Then
                    Throw New Exception("Old Version!")
                ElseIf line = "~|B2|~" Then
                    Throw New Exception("Old Version!")
                ElseIf line = "~|B3|~" Then
                    Dim result As CoverRun = Nothing
                    Try
                        result = ParseV3(sr)
                    Catch
                        Do While sr.ReadLine() <> "~|E|~"
                        Loop
                    End Try

                    If result IsNot Nothing Then
                        runs.Add(result)
                    End If
                End If
            Loop
        End Using
        Return runs
    End Function


	Shared Function ParseV3(ByVal sr As StreamReader) As CoverRun
		Dim coverRun As New CoverRun()
		coverRun.Program = CoverRun.ExtractField(sr.ReadLine(), "A")
		coverRun.CommandLine = CoverRun.ExtractField(sr.ReadLine(), "C")
		coverRun.LogTime = DateTime.ParseExact(CoverRun.ExtractField(sr.ReadLine(), "D"), "yyyyMMdd`HHmmss", CultureInfo.InvariantCulture)
		coverRun.VersionTag = CoverRun.ExtractField(sr.ReadLine(), "T")

		coverRun.MachineName = CoverRun.ExtractField(sr.ReadLine(), "M")
		coverRun.OS = CoverRun.ExtractField(sr.ReadLine(), "O")
		coverRun.ProcCount = CoverRun.ExtractField(Of Integer)(sr.ReadLine(), "P")

		coverRun.Data = New List(Of RunData)()

		CoverRun.EatUntil(sr, "SP")

		Dim line As String
		line = sr.ReadLine()
		Do While line <> "~|E|~"
			' 548|Entered
			Dim parts() As String = line.Split("|"c)
			Debug.Assert(parts.Length = 3)

			coverRun.Data.Add(New RunData() With {.SyncId = Integer.Parse(parts(0)), .Hit = Long.Parse(parts(1)), .Contend = Long.Parse(parts(2))})
			line = sr.ReadLine()
		Loop
		Return coverRun

	End Function

	Public Shared Function ExtractField(Of T1)(ByVal line As String, ByVal fieldName As String) As T1
		Dim asString As String = ExtractField(line, fieldName)
		Return CType(Convert.ChangeType(asString, GetType(T1)), T1)
	End Function

	Public Shared Sub EatUntil(ByVal sr As StreamReader, ByVal p As String)
		Do While sr.ReadLine() <> p & "|"
		Loop
	End Sub

	Public Shared Function ExtractField(ByVal line As String, ByVal fieldName As String) As String
		Dim parts() As String = line.Split(New Char() { "|"c }, 2)

		If parts.Length <> 2 Then
			Throw New Exception(String.Format("ExtractField invalid line={0}", line))
		ElseIf parts(0) <> fieldName Then
			Throw New Exception(String.Format("ExtractField expected={0} actual={1}", fieldName, parts(0)))
		End If

		Return parts(1)
	End Function

End Class ' class
