Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.Collections.ObjectModel
Imports System.IO
Imports System.Linq
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Data
Imports System.Windows.Media

Namespace SyncView

	Public Class LineData
		Private privateFiles As ObservableCollection(Of String)
		Public Property Files() As ObservableCollection(Of String)
			Get
				Return privateFiles
			End Get
			Set(ByVal value As ObservableCollection(Of String))
				privateFiles = value
			End Set
		End Property
	End Class

	Partial Public Class Window1
		Inherits Window

		Private mData As LineData

		Public Sub New()
			InitializeComponent()

			mData = New LineData()
			mData.Files = New ObservableCollection(Of String)()

			DataContext = mData
		End Sub

		Private Sub Button_NextClick(ByVal sender As Object, ByVal e As RoutedEventArgs)
			FindNext(1)
		End Sub

		Private Sub FindNext(ByVal dir As Integer)
			Dim i As Integer = If(lstSource.SelectedIndex > 0, lstSource.SelectedIndex + dir, 0)

			Do While i >= 0 AndAlso i < lstSource.Items.Count
				Dim f As LineInfo = CType(lstSource.Items(i), LineInfo)
				If f.Priority <> 0 Then
					Exit Do
				End If
				i += dir
			Loop

			If i >= 0 AndAlso i < lstSource.Items.Count Then
				lstSource.SelectedIndex = i
				lstSource.ScrollIntoView(lstSource.SelectedItem)
			End If
		End Sub

		Private Sub Button_PrevClick(ByVal sender As Object, ByVal e As RoutedEventArgs)
			FindNext(-1)
		End Sub

		Private Sub ListBox_SelectionChanged(ByVal sender As Object, ByVal e As System.Windows.Controls.SelectionChangedEventArgs)
		End Sub

		Private Sub lstSource_DataContextChanged(ByVal sender As Object, ByVal e As DependencyPropertyChangedEventArgs)
			TryCast(((TryCast(VisualTreeHelper.GetChild(lstSource, 0), Border)).Child), ScrollViewer).ScrollToVerticalOffset(0)
		End Sub

		Private Sub Button_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)

			CoverFile = GetCoverFile()
			MapFile = GetMapFile(CoverFile)

			' Process open file dialog box results
			If File.Exists(CoverFile) Then

				For Each fileName In CoverMap.FromFile(MapFile).Select(Function(x) x.FileName).Distinct()
					If File.Exists(fileName) Then
						mData.Files.Add(fileName)
					End If
				Next fileName
			End If

			DataContext = mData
		End Sub

		Private Shared Function GetMapFile(ByVal probePath As String) As String
            Dim f = Directory.GetFiles(Path.GetDirectoryName(probePath), "cc.map.*.txt").FirstOrDefault()

            If File.Exists(f) Then
                Return f
            End If
			Dim dlg As Microsoft.Win32.OpenFileDialog

			' Configure open file dialog box
			dlg = New Microsoft.Win32.OpenFileDialog()
			dlg.InitialDirectory = Path.GetDirectoryName(probePath)
			dlg.Title = "Open Coverage Map File"
			dlg.FileName = ""
			dlg.DefaultExt = ".txt" ' Default file extension
			dlg.Filter = "Coverage Map(cc.cov.*.txt)|cc.map.*.txt"

			If dlg.ShowDialog() = True Then
				Return dlg.FileName
			Else
				Return ""
			End If
		End Function

		Private Shared Function GetCoverFile() As String
			Dim dlg As Microsoft.Win32.OpenFileDialog

			' Configure open file dialog box
			dlg = New Microsoft.Win32.OpenFileDialog()
			dlg.InitialDirectory = Environment.CurrentDirectory
			dlg.Title = "Coverage Data File"
			dlg.FileName = ""
			dlg.DefaultExt = ".txt" ' Default file extension
			dlg.Filter = "Coverage Data(cc.cov.*.txt)|cc.cov.*.txt"

			If dlg.ShowDialog() = True Then
				Return dlg.FileName
			Else
				Return ""
			End If
		End Function

		' HACK: Pass this into the converters!
		Friend Shared MapFile As String ' = "cc.map.DEMO.txt";
		Friend Shared CoverFile As String ' = "cc.cov.FOO.DWEEZIL.txt";

	End Class ' class

	Public Class LineInfo

		Private privateText As String
		Public Property Text() As String
			Get
				Return privateText
			End Get
			Set(ByVal value As String)
				privateText = value
			End Set
		End Property
		Private privateLineNumber As Integer
		Public Property LineNumber() As Integer
			Get
				Return privateLineNumber
			End Get
			Set(ByVal value As Integer)
				privateLineNumber = value
			End Set
		End Property
		Private privatePriority As Integer
		Public Property Priority() As Integer
			Get
				Return privatePriority
			End Get
			Set(ByVal value As Integer)
				privatePriority = value
			End Set
		End Property
		Private privateSyncId As Integer
		Public Property SyncId() As Integer
			Get
				Return privateSyncId
			End Get
			Set(ByVal value As Integer)
				privateSyncId = value
			End Set
		End Property

		Public Overrides Function ToString() As String
			Return "MyFoo+ToString(TODO)"
		End Function

	End Class ' class

End Namespace ' namespace
